#include "bella_sdk/bella_scene.h"

using namespace dl;
using namespace bella_sdk;

// We will use the dl_core main helper here. This gives us a helpful Args instance to use, and
// also hides the confusing details of dealing with main vs. WinMain on windows, and gives us
// utf8-encoded args when the application is unicode.
//
#include "dl_core/dl_main.inl"
int DL_main(Args& /*args*/)
{
    // Here, we show that we can use a scene and nodes regardless that we have no implementation
    // for them. We will get nothing but default values out, but we will be able to write any of
    // the code typically written using these types.
    //
    // Typically you will use this SDK for writing your own node; for example say you want to make
    // your own custom type of checker node, you would implement the various C functions of a node,
    // and those would be called by Bella when it needs you to provide values. When it does call
    // your code, passing an INode*, you will wrap that INode* in a bella_sdk::Node, making it
    // easier to use, and ensuring it cannot be deleted while you are using it.
    //
    // And as long as we (the Bella devs) never remove methods from the various interfaces, your
    // custom node library will continue to work in any future version of Bella.

    logBanner("Bella Node SDK (version: %s, build date: %llu)",
        bellaSdkVersion().toString().buf(),
        bellaSdkBuildDate()
    );

    Scene scene;

    auto def = scene.defineNode("myNodeType");
    auto obj = def.createInput("obj", AttrType_Object);
    obj.createInput("int", AttrType_Int);
    obj.createInput("real", AttrType_Real);

    auto node = scene.createNode("myNodeType");
    auto ival = node["obj"]["int"].asInt();
    auto rval = node["obj"]["real"].asReal();

    logInfo("Show that we have used much of the API, but created nothing:");
    logInfo("  def is null: %s", String(def == nullptr).buf());
    logInfo("  node is null: %s", String(node == nullptr).buf());
    logInfo("  value of ival: %d", ival);
    logInfo("  value of rval: %f", rval);

    return 0;
}
